class Chord
  attr_reader :name
  def initialize(name)
    @name = name
    validate!
  end

  def notes
    @notes ||= find_notes
  end

  def ==(other_chord)
    notes == other_chord.notes
  end

  def first
    notes[0]
  end

  def third
    notes[1]
  end

  def fifth
    notes[2]
  end

  def to_s
    notes.map(&:to_s).join(" ")
  end

  private
  def find_notes
    note_name, suffix = parse_name
    root = Note.find(note_name)
    steps = CHORD_RULES[suffix]
    steps.map { |dist| root + dist }
  end

  def validate!
    raise ArgumentError.new("This chord is not currently supported") unless valid?
  end

  def parse_name
    match = name.match(/^[a-gA-G][#!b_]?/)
    [match[0], match.post_match]
  end

  def valid?
    !!(name =~ /^[a-gA-G][#!b_]?[a-zA-z]*7*$/)
  end
end

CHORD_RULES = {
  ""            => [0, 4, 7],
  "maj"         => [0, 4, 7],
  "major"       => [0, 4, 7],
  "maj7"        => [0, 4, 7, 11],
  "major7"      => [0, 4, 7, 11],
  "m"           => [0, 3, 7],
  "min"         => [0, 3, 7],
  "minor"       => [0, 3, 7],
  "min7"        => [0, 3, 7, 10],
  "minor7"      => [0, 3, 7, 10],
  "m7"          => [0, 3, 7, 10],
  "aug"         => [0, 4, 8],
  "augmented"   => [0, 4, 8],
  "aug7"        => [0, 4, 8, 10],
  "augmented7"  => [0, 4, 8, 10],
  "dim"         => [0, 3, 6],
  "diminished"  => [0, 3, 6],
  "dim7"        => [0, 3, 6, 9],
  "diminished7" => [0, 3, 6, 9],
  "7"           => [0, 4, 7, 10],
  "dominant7"   => [0, 4, 7, 10]
}
