class Note
  ACCIDENTALS = {
    nil => "",
    "#" => "#",
    "!" => "#",
    "b" => "♭",
    "_" => "♭"
  }

  def self.find(name)
    JOIN_SCALE.find_note(name)
  end

  MUTED = "x".freeze

  attr_reader :name, :position

  alias_method :to_s, :name

  def initialize(name, position = nil)
    @name = name == MUTED ? name : convert_name(name)
    @position = position
  end

  def ==(other_note)
    position == other_note.position
  end

  def +(steps)
    SHARPS_SCALE[position + steps]
  end

  def muted?
    name == MUTED
  end

  private
  def convert_name(name)
    raise ArgumentError.new("You must enter a valid note name.") unless valid?(name)
    name[0].upcase + ACCIDENTALS[name[1]]
  end

  def valid?(name)
    !!(name =~ /^[a-gA-G][#!b_]?$/)
  end
end
